@@ -106,7 +106,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             @param album_artist_ids as [int]
             @param genre_ids as [int]
             @param year as int
-            @param timestmap as int
+            @param timestamp as int
             @commit needed
         """
         if album_artist_ids:
@@ -132,10 +132,10 @@ class CollectionScanner(GObject.GObject, TagReader):
     def save_track(self, genres, artists, a_sortnames, mb_artist_id,
                    album_artists, aa_sortnames, mb_album_artist_id,
                    album_name, mb_album_id, uri, album_loved, album_pop,
-                   album_rate, album_mtime, title, duration, tracknumber,
-                   discnumber, discname, year, timestamp, track_mtime,
-                   track_pop, track_rate, track_loved, track_ltime,
-                   mb_track_id, bpm):
+                   album_rate, album_synced, album_mtime, title, duration,
+                   tracknumber, discnumber, discname, year, timestamp,
+                   track_mtime, track_pop, track_rate, track_loved,
+                   track_ltime, mb_track_id, bpm):
         """
             Add track to DB
             @param genres as str/None
@@ -151,6 +151,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             @param album_loved as int
             @param album_pop as int
             @param album_rate as int
+            @param album_synced as int
             @param album_mtime as int
             @param title as str
             @param duration as int
@@ -192,7 +193,8 @@ class CollectionScanner(GObject.GObject, TagReader):
         (album_added, album_id) = self.add_album(album_name, mb_album_id,
                                                  album_artist_ids,
                                                  uri, album_loved, album_pop,
-                                                 album_rate, album_mtime)
+                                                 album_rate, album_synced,
+                                                 album_mtime)
         if genres is None:
             genre_ids = [Type.WEB]
         else:
@@ -213,7 +215,9 @@ class CollectionScanner(GObject.GObject, TagReader):
                           genre_ids, year, timestamp)
         SqlCursor.commit(App().db)
         for genre_id in genre_ids:
-            GLib.idle_add(self.emit, "genre-updated", genre_id, True)
+            # Be sure to not send Type.WEB
+            if genre_id >= 0:
+                GLib.idle_add(self.emit, "genre-updated", genre_id, True)
         if album_added:
             GLib.idle_add(self.emit, "album-updated", album_id, True)
         return (track_id, album_id)
@@ -221,11 +225,9 @@ class CollectionScanner(GObject.GObject, TagReader):
     def update_track(self, track_id, artist_ids, genre_ids):
         """
             Set track artists/genres
-            @param track id as int
-            @param artist ids as [int]
-            @param genre ids as [int]
-            @param mtime as int
-            @param popularity as int
+            @param track_id as int
+            @param artist_ids as [int]
+            @param genre_ids as [int]
             @commit needed
         """
         # Set artists/genres for track
@@ -258,6 +260,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             album_pop = App().albums.get_popularity(album_id)
             album_rate = App().albums.get_rate(album_id)
             album_loved = App().albums.get_loved(album_id)
+            album_synced = App().albums.get_synced(album_id)
             # Force genre for album
             App().albums.set_genre_ids(album_id, genre_ids)
             if backup:
@@ -265,7 +268,8 @@ class CollectionScanner(GObject.GObject, TagReader):
                 name = f.get_basename()
                 self.__history.add(name, duration, track_pop, track_rate,
                                    track_ltime, album_mtime, track_loved,
-                                   album_loved, album_pop, album_rate)
+                                   album_loved, album_pop, album_rate,
+                                   album_synced)
             App().tracks.remove(track_id)
             App().albums.clean()
             App().genres.clean()
@@ -328,8 +332,6 @@ class CollectionScanner(GObject.GObject, TagReader):
         self.emit("scan-finished", modifications)
         # Update max count value
         App().albums.update_max_count()
-        if App().settings.get_value("artist-artwork"):
-            App().art.cache_artists_info()
 
     def __add_monitor(self, dirs):
         """
@@ -370,11 +372,12 @@ class CollectionScanner(GObject.GObject, TagReader):
                        ";".join(data["album_artists"]),
                        "", "", data["album_name"], "", data["uri"],
                        data["album_loved"], data["album_popularity"],
-                       data["album_rate"], -1, data["title"], data["duration"],
-                       data["tracknumber"], data["discnumber"],
-                       data["discname"], data["year"], data["timestamp"], -1,
-                       data["track_popularity"], data["track_rate"],
-                       data["track_loved"], 0, "", 0)
+                       data["album_rate"], 0, -1, data["title"],
+                       data["duration"], data["tracknumber"],
+                       data["discnumber"], data["discname"], data["year"],
+                       data["timestamp"], -1, data["track_popularity"],
+                       data["track_rate"], data["track_loved"], 0, "", 0)
+            infos.close(None)
         except Exception as e:
             Logger.error("CollectionScanner::__import_web_tracks(): %s", e)
 
@@ -421,6 +424,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                         else:
                             mtime = get_mtime(info)
                             files.append((mtime, child_uri))
+                    infos.close(None)
                 # Only happens if files passed as args
                 else:
                     mtime = get_mtime(info)
@@ -576,6 +580,7 @@ class CollectionScanner(GObject.GObject, TagReader):
         aa_sortnames = self.get_album_artist_sortnames(tags)
         album_artists = self.get_album_artists(tags)
         album_name = self.get_album_name(tags)
+        album_synced = 0
         mb_album_id = self.get_mb_album_id(tags)
         mb_track_id = self.get_mb_track_id(tags)
         mb_artist_id = self.get_mb_artist_id(tags)
@@ -617,7 +622,8 @@ class CollectionScanner(GObject.GObject, TagReader):
         if track_id is None:
             (track_pop, track_rate, track_ltime,
              album_mtime, track_loved, album_loved,
-             album_pop, album_rate) = self.__history.get(name, duration)
+             album_pop, album_rate, album_synced) = self.__history.get(
+                name, duration)
         # Delete track and restore from it
         else:
             (track_pop, track_rate, track_ltime,
@@ -634,8 +640,8 @@ class CollectionScanner(GObject.GObject, TagReader):
                    genres, artists, a_sortnames, mb_artist_id,
                    album_artists, aa_sortnames, mb_album_artist_id,
                    album_name, mb_album_id, uri, album_loved, album_pop,
-                   album_rate, album_mtime, title, duration, tracknumber,
-                   discnumber, discname, year, timestamp, track_mtime,
-                   track_pop, track_rate, track_loved, track_ltime,
-                   mb_track_id, bpm)
+                   album_rate, album_synced, album_mtime, title, duration,
+                   tracknumber, discnumber, discname, year, timestamp,
+                   track_mtime, track_pop, track_rate, track_loved,
+                   track_ltime, mb_track_id, bpm)
         return track_id
