@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk
+from gi.repository import Gtk, GLib
 
 from lollypop.define import App, ViewType
 from lollypop.widgets_utils import Popover
@@ -26,8 +26,13 @@ class CurrentPopover(Popover):
             Init popover
         """
         Popover.__init__(self)
-        self.__view = App().window.container.get_view_current(
-            ViewType.POPOVER | ViewType.DND)
+        # No DND until https://gitlab.gnome.org/GNOME/gtk/issues/2006 is fixed
+        wayland = GLib.environ_getenv(GLib.get_environ(), "WAYLAND_DISPLAY")
+        if wayland:
+            view_type = ViewType.POPOVER
+        else:
+            view_type = ViewType.POPOVER | ViewType.DND
+        self.__view = App().window.container.get_view_current(view_type)
         self.__view.show()
         self.set_position(Gtk.PositionType.BOTTOM)
         self.connect("map", self.__on_map)
