@@ -14,6 +14,7 @@ from gi.repository import GLib, Gio, Gst, GObject
 
 from time import sleep
 from re import match
+from random import shuffle
 import json
 
 from lollypop.logger import Logger
@@ -190,10 +191,10 @@ class MtpSync(GObject.Object):
                                       ! oggmux",
                   "convert_flac": " ! flacenc",
                   "convert_aac": " ! faac bitrate=%s ! mp4mux"}
-    _GST_ENCODER = {"convert_mp3": "lamemp3enc",
-                    "convert_ogg": "vorbisenc",
-                    "convert_flac": "flacenc",
-                    "convert_aac": "faac"}
+    _GST_ENCODER = {"convert_mp3": ["lamemp3enc", "id3v2mux"],
+                    "convert_ogg": ["vorbisenc", "oggmux"],
+                    "convert_flac": ["flacenc"],
+                    "convert_aac": ["faac", "mp4mux"]}
 
     def __init__(self):
         """
@@ -201,7 +202,6 @@ class MtpSync(GObject.Object):
         """
         GObject.Object.__init__(self)
         self.__cancellable = Gio.Cancellable()
-        self.__cancellable.cancel()
         self.__errors_count = 0
         self.__on_mtp_files = []
         self.__last_error = ""
@@ -216,103 +216,99 @@ class MtpSync(GObject.Object):
             @param encoder as str
             @return bool
         """
-        if Gst.ElementFactory.find(self._GST_ENCODER[encoder]):
-            return True
-        return False
+        ok = True
+        for item in self._GST_ENCODER[encoder]:
+            if not Gst.ElementFactory.find(item):
+                ok = False
+        return ok
 
-    def sync(self, uri):
+    def sync(self, uri, index):
         """
             Sync playlists with device. If playlists contains Type.NONE,
             sync albums marked as to be synced
             @param uri as str
+            @param index as int => device index
         """
         try:
             self.__cancellable = Gio.Cancellable()
             self.__uri = uri
-            self.__on_mtp_files = self.__get_track_files()
             self.__convert_bitrate = App().settings.get_value(
                 "convert-bitrate").get_int32()
             self.__errors_count = 0
-            # For progress bar
             self.__total = 0
             self.__done = 0
-            playlists = []
+            tracks = []
 
-            Logger.debug("Get new tracks before sync")
+            Logger.info("Getting tracks to sync")
             # New tracks for synced albums
-            album_ids = App().albums.get_synced_ids()
+            album_ids = App().albums.get_synced_ids(0)
+            album_ids += App().albums.get_synced_ids(index)
             for album_id in album_ids:
                 album = Album(album_id)
-                self.__total += len(album.track_ids)
+                tracks += album.tracks
             # New tracks for playlists
-            playlist_ids = App().playlists.get_synced_ids()
+            playlist_ids = App().playlists.get_synced_ids(0)
+            playlist_ids += App().playlists.get_synced_ids(index)
             for playlist_id in playlist_ids:
-                name = App().playlists.get_name(playlist_id)
-                playlists.append(escape(name))
                 if App().playlists.get_smart(playlist_id):
                     request = App().playlists.get_smart_sql(playlist_id)
-                    track_ids = App().db.execute(request)
+                    for track_id in App().db.execute(request):
+                        tracks.append(Track(track_id))
                 else:
-                    track_ids = App().playlists.get_track_ids(playlist_id)
-                self.__total += len(track_ids)
+                    for track_id in App().playlists.get_track_ids(playlist_id):
+                        tracks.append(Track(track_id))
 
-            Logger.debug("Get old tracks")
-            self.__total += len(self.__on_mtp_files)
+            Logger.info("Getting URIs to copy")
+            uris = self.__get_uris_to_copy(tracks)
+            shuffle(uris)
+            self.__total = len(uris) + 2
 
-            # Copy new tracks to device
+            Logger.info("Deleting old files")
             if not self.__cancellable.is_cancelled():
-                Logger.debug("Sync albums")
-                self.__sync_albums()
-                Logger.debug("Sync playlists")
-                self.__sync_playlists(playlist_ids)
+                self.__delete_old_uris(uris)
 
-            # Remove old tracks from device
+            Logger.info("Copying files")
+            for (src_uri, dst_uri) in uris:
+                try:
+                    if self.__cancellable.is_cancelled():
+                        break
+                    self.__copy_file(src_uri, dst_uri)
+                    self.__done += 1
+                    GLib.idle_add(self.emit, "sync-progress",
+                                  self.__done / self.__total)
+                except Exception as e:
+                    Logger.error("MtpSync::__copy_files(): %s", e)
+            Logger.debug("Writing playlists")
             if not self.__cancellable.is_cancelled():
-                Logger.debug("Remove from device")
-                self.__remove_from_device(playlist_ids)
-
-            # Remove empty dirs
+                self.__write_playlists(playlist_ids)
+            GLib.idle_add(self.emit, "sync-progress",
+                          self.__done / self.__total + 1)
+            Logger.debug("Creating unsync")
             if not self.__cancellable.is_cancelled():
-                Logger.debug("Remove empty dirs")
-                self.__remove_empty_dirs()
-
-            if not self.__cancellable.is_cancelled():
-                Logger.debug("Remove old playlists")
-                # Remove old playlists
-                d = Gio.File.new_for_uri(self.__uri)
-                infos = d.enumerate_children(
-                    "standard::name",
-                    Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
-                    None)
-                for info in infos:
-                    name = info.get_name()
-                    if name.endswith(".m3u") and name[:-4] not in playlists:
-                        f = infos.get_child(info)
-                        self.__retry(f.delete, (None,))
-
-            Logger.debug("Create unsync")
-            d = Gio.File.new_for_uri(self.__uri + "/unsync")
-            if not d.query_exists():
-                self.__retry(d.make_directory_with_parents, (None,))
+                d = Gio.File.new_for_uri(self.__uri + "/unsync")
+                if not d.query_exists():
+                    d.make_directory_with_parents()
+            GLib.idle_add(self.emit, "sync-progress",
+                          self.__done / self.__total + 2)
         except Exception as e:
             Logger.error("MtpSync::__sync(): %s" % e)
         finally:
-            Logger.debug("Save sync db")
-            self.__mtp_syncdb.save()
-            self.__cancellable.cancel()
+            Logger.info("Save sync db")
+            if not self.__cancellable.is_cancelled():
+                self.__mtp_syncdb.save()
+            self.cancel()
             if self.__errors_count != 0:
                 Logger.debug("Sync errors")
                 GLib.idle_add(self.emit, "sync-errors", self.__last_error)
             Logger.debug("Sync finished")
             GLib.idle_add(self.emit, "sync-finished")
 
-    @property
-    def cancellable(self):
+    def cancel(self):
         """
-            Get cancellable
-            @return Gio.Cancellable
+            Cancel sync
         """
-        return self.__cancellable
+        Logger.info("MtpSync::cancel()")
+        self.__cancellable.cancel()
 
     @property
     def db(self):
@@ -322,85 +318,131 @@ class MtpSync(GObject.Object):
         return self.__mtp_syncdb
 
 ############
-# Private  #
+# PRIVATE  #
 ############
-    def __retry(self, func, args):
+    def __get_album_on_device_uri(self, track):
         """
-            Try to execute func and handle errors
-            @param func as function
-            @param args as tuple
+            Get on device URI for album
+            @param track as Track
+            @return URI as str
         """
-        # Max allowed errors
-        if self.__errors_count > 5:
-            self.__cancellable.cancel()
-            return
-        try:
-            func(*args)
-        except Exception as e:
-            Logger.error("MtpSync::_retry(%s, %s): %s" % (func, args, e))
-            self.__last_error = e
-            self.__errors_count += 1
-            sleep(1)
+        album_name = track.album_name.lower()
+        is_compilation = track.album.artist_ids[0] == Type.COMPILATIONS
+        if is_compilation:
+            string = escape(album_name)
+        else:
+            artists = ", ".join(track.album.artists).lower()
+            string = escape("%s_%s" % (artists, album_name))
+        return GLib.uri_escape_string(string[:100], None, True)
 
-    def __remove_empty_dirs(self):
+    def __get_uris_to_copy(self, tracks):
         """
-            Delete empty dirs
+            Get on device URI for all tracks
+            @param tracks as [Track]
         """
-        to_delete = []
-        dir_uris = [self.__uri]
-        try:
-            # First get all directories
-            while dir_uris:
-                if self.__cancellable.is_cancelled():
-                    break
-                uri = dir_uris.pop(0)
-                d = Gio.File.new_for_uri(uri)
-                infos = d.enumerate_children(
+        uris = []
+        art_uris = []
+        for track in tracks:
+            f = Gio.File.new_for_uri(track.uri)
+            album_device_uri = "%s/%s" % (self.__uri,
+                                          self.__get_album_on_device_uri(
+                                            track))
+            album_local_uri = f.get_parent().get_uri()
+            src_uri = "%s/%s" % (album_local_uri,
+                                 GLib.uri_escape_string(f.get_basename(),
+                                                        None,
+                                                        True))
+            dst_uri = "%s/%s" % (album_device_uri, escape(f.get_basename()))
+            (convertion_needed,
+             dst_uri) = self.__is_convertion_needed(src_uri, dst_uri)
+            uris.append((src_uri, dst_uri))
+            art_uri = App().art.get_album_artwork_uri(track.album)
+            if art_uri is not None:
+                art_filename = Gio.File.new_for_uri(art_uri).get_basename()
+                art_uris.append((art_uri,
+                                 "%s/%s" % (album_device_uri,
+                                            escape(art_filename))))
+        return uris + art_uris
+
+    def __write_playlists(self, playlist_ids):
+        """
+            Write playlists on disk
+            @param playlist_ids as [int]
+        """
+        for playlist_id in playlist_ids:
+            if self.__cancellable.is_cancelled():
+                break
+            try:
+                name = escape(App().playlists.get_name(playlist_id))
+                dst_uri = "%s/%s.m3u" % (self.__uri, name)
+                if App().playlists.get_smart(playlist_id):
+                    request = App().playlists.get_smart_sql(playlist_id)
+                    track_ids = App().db.execute(request)
+                else:
+                    track_ids = App().playlists.get_track_ids(playlist_id)
+                # Create playlist
+                m3u = Gio.File.new_for_path("/tmp/lollypop_%s.m3u" % name)
+                content = "#EXTM3U\n"
+                for track_id in track_ids:
+                    track = Track(track_id)
+                    if self.__cancellable.is_cancelled():
+                        break
+                    f = Gio.File.new_for_uri(track.uri)
+                    filename = f.get_basename()
+                    album_uri = self.__get_album_on_device_uri(track)
+                    uri = "%s/%s" % (album_uri, filename)
+                    (convertion_needed,
+                     uri) = self.__is_convertion_needed(track.uri, uri)
+                    content += "%s\n" % uri
+                    m3u.replace_contents(
+                                    content.encode("utf-8"),
+                                    None,
+                                    False,
+                                    Gio.FileCreateFlags.REPLACE_DESTINATION,
+                                    self.__cancellable)
+                    dst = Gio.File.new_for_uri(dst_uri)
+                    Logger.debug("MtpSync::__write_playlists(): %s" % dst_uri)
+                    m3u.move(dst, Gio.FileCopyFlags.OVERWRITE, None, None)
+            except Exception as e:
+                Logger.error("MtpSync::__write_playlists(): %s", e)
+
+    def __delete_old_uris(self, uris):
+        """
+            Delete old URIs from device
+            @param uris as [str]
+        """
+        on_device_uris = self.__on_device_uris()
+        for (src_uri, dst_uri) in uris:
+            dst = Gio.File.new_for_uri(dst_uri)
+            if dst.get_uri() in on_device_uris:
+                on_device_uris.remove(dst.get_uri())
+        for uri in on_device_uris:
+            if self.__cancellable.is_cancelled():
+                break
+            try:
+                f = Gio.File.new_for_uri(uri)
+                parent = f.get_parent()
+                f.delete(self.__cancellable)
+                self.__mtp_syncdb.delete_uri(uri)
+                infos = parent.enumerate_children(
                     "standard::name,standard::type",
                     Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
                     None)
-                for info in infos:
-                    if self.__cancellable.is_cancelled():
-                        break
-                    if info.get_file_type() == Gio.FileType.DIRECTORY:
-                        if info.get_name() != "unsync":
-                            f = infos.get_child(info)
-                            # We need to check for dir to be empty
-                            # On some device, Gio.File.delete() remove
-                            # non empty directories #828
-                            subinfos = f.enumerate_children(
-                                "standard::name,standard::type",
-                                Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
-                                None)
-                            subfiles = False
-                            for info in subinfos:
-                                subfiles = True
-                                dir_uris.append(f.get_uri())
-                                break
-                            if not subfiles:
-                                to_delete.append(f.get_uri())
-            # Then delete
-            for d in to_delete:
-                if self.__cancellable.is_cancelled():
-                    break
-                d = Gio.File.new_for_uri(d)
-                try:
-                    d.delete()
-                except:
-                    pass
-        except Exception as e:
-            Logger.error("MtpSync::__remove_empty_dirs(): %s, %s" % (e, uri))
+                if len(list(infos)) == 0:
+                    parent.delete(self.__cancellable)
+            except Exception as e:
+                Logger.error("MtpSync::__delete_old_uris(): %s", e)
 
-    def __get_track_files(self):
+    def __on_device_uris(self):
         """
-            Return files in self.__uri/tracks
+            Get URIS on device
             @return [str]
         """
         children = []
         dir_uris = [self.__uri]
         d = Gio.File.new_for_uri(self.__uri)
         if not d.query_exists():
-            self.__retry(d.make_directory_with_parents, (None,))
+            d.make_directory_with_parents(None)
         while dir_uris:
             if self.__cancellable.is_cancelled():
                 break
@@ -422,82 +464,57 @@ class MtpSync(GObject.Object):
                         if info.get_name() == "lollypop-sync.db":
                             continue
                         f = infos.get_child(info)
-                        if not f.get_uri().endswith(".m3u"):
-                            children.append(f.get_uri())
+                        children.append(f.get_uri())
             except Exception as e:
                 Logger.error("MtpSync::__get_track_files(): %s, %s" % (e, uri))
         return children
 
-    def __sync_track_id(self, track):
+    def __is_convertion_needed(self, src_uri, dst_uri):
         """
-            Sync track to device
-            @param track as Track
-            @return (str, str, str, bool)
+            Check if convertion is needed
+            @param src_uri as str
+            @param dst_uri as str
+            @return (bool, str) => Is needed and new dst URI
         """
-        Logger.debug("MtpSync::__sync_track_id(): %s" % track.uri)
-        album_name = escape(track.album_name.lower())
-        is_compilation = track.album.artist_ids[0] == Type.COMPILATIONS
-        if is_compilation:
-            artists = None
-            on_device_album_uri = "%s/%s" %\
-                                  (self.__uri,
-                                   album_name)
-        else:
-            artists = escape(", ".join(track.album.artists).lower())
-            on_device_album_uri = "%s/%s_%s" %\
-                                  (self.__uri,
-                                   artists,
-                                   album_name)
-
-        d = Gio.File.new_for_uri(on_device_album_uri)
-        if not d.query_exists():
-            self.__retry(d.make_directory_with_parents, (None,))
-        # Copy album art
-        art = App().art.get_album_artwork_uri(track.album)
-        Logger.debug("MtpSync::__sync_track_id(): %s" % art)
-        if art is not None:
-            src_art = Gio.File.new_for_uri(art)
-            art_uri = "%s/cover.jpg" % on_device_album_uri
-            # To be sure to get uri correctly escaped for Gio
-            f = Gio.File.new_for_uri(art_uri)
-            dst_art = Gio.File.new_for_uri(art_uri)
-            if dst_art.get_uri() in self.__on_mtp_files:
-                self.__on_mtp_files.remove(dst_art.get_uri())
-            if not dst_art.query_exists():
-                self.__retry(src_art.copy,
-                             (dst_art, Gio.FileCopyFlags.OVERWRITE,
-                              None, None))
-        f = Gio.File.new_for_uri(track.uri)
-        track_name = escape(f.get_basename())
-        # Check extension, if not mp3, convert
-        m = match(r".*(\.[^.]*)", track.uri)
+        # Check extension for convertion
+        m = match(r".*(\.[^.]*)", src_uri)
         ext = m.group(1)
         convert_ext = self.__EXTENSION[self.__mtp_syncdb.encoder]
-        if (convert_ext is not None and
+        if ext.lower() in [".png", ".jpg", ".jpeg", "gif"]:
+            convertion_needed = False
+        elif (convert_ext is not None and
                 ext != convert_ext) or self.__mtp_syncdb.normalize:
             convertion_needed = True
-            track_name = track_name.replace(ext, convert_ext)
+            dst_uri = dst_uri.replace(ext, convert_ext)
         else:
             convertion_needed = False
-        src_track = Gio.File.new_for_uri(track.uri)
-        info = src_track.query_info("time::modified",
-                                    Gio.FileQueryInfoFlags.NONE,
-                                    None)
-        # Prefix track with mtime to make sure updating it later
+        return (convertion_needed, dst_uri)
+
+    def __copy_file(self, src_uri, dst_uri):
+        """
+            Copy source to destination
+            @param src_uri as str
+            @param dst_uri as str
+        """
+        src = Gio.File.new_for_uri(src_uri)
+        (convertion_needed,
+         dst_uri) = self.__is_convertion_needed(src_uri, dst_uri)
+        dst = Gio.File.new_for_uri(dst_uri)
+        parent = dst.get_parent()
+        if not parent.query_exists():
+            parent.make_directory_with_parents()
+        info = src.query_info("time::modified",
+                              Gio.FileQueryInfoFlags.NONE,
+                              None)
         mtime = info.get_attribute_uint64("time::modified")
-        dst_uri = "%s/%s" % (on_device_album_uri,
-                             track_name)
-        dst_track = Gio.File.new_for_uri(dst_uri)
-        if dst_track.get_uri() in self.__on_mtp_files:
-            self.__on_mtp_files.remove(dst_track.get_uri())
-        if not dst_track.query_exists() or\
-            self.__mtp_syncdb.get_mtime(
-                dst_track.get_uri()) < mtime:
+        if not dst.query_exists() or\
+                self.__mtp_syncdb.get_mtime(dst_uri) < mtime:
+            Logger.debug("MtpSync::__copy_file(): %s -> %s"
+                         % (src_uri, dst_uri))
             if convertion_needed:
-                convert_uri = "file:///tmp/%s" % track_name
+                convert_uri = "file:///tmp/lollypop_convert"
                 convert_file = Gio.File.new_for_uri(convert_uri)
-                pipeline = self.__convert(src_track,
-                                          convert_file)
+                pipeline = self.__convert(src, convert_file)
                 # Check if encoding is finished
                 if pipeline is not None:
                     bus = pipeline.get_bus()
@@ -511,114 +528,16 @@ class MtpSync(GObject.Object):
                     pipeline.set_state(Gst.State.PAUSED)
                     pipeline.set_state(Gst.State.READY)
                     pipeline.set_state(Gst.State.NULL)
-                    self.__retry(
-                        convert_file.move,
-                        (dst_track, Gio.FileCopyFlags.OVERWRITE,
-                         None, None))
+                    convert_file.move(
+                        dst, Gio.FileCopyFlags.OVERWRITE, None, None)
                     # To be sure
                     try:
                         convert_file.delete(None)
                     except:
                         pass
             else:
-                self.__retry(src_track.copy,
-                             (dst_track, Gio.FileCopyFlags.OVERWRITE,
-                              None, None))
-            self.__mtp_syncdb.set_mtime(dst_track.get_uri(), mtime)
-        self.__done += 1
-        GLib.idle_add(self.emit, "sync-progress",
-                      self.__done / self.__total)
-        return (track_name, artists, album_name, is_compilation)
-
-    def __sync_albums(self):
-        """
-            Sync albums to device
-        """
-        album_ids = App().albums.get_synced_ids()
-        for album_id in album_ids:
-            album = Album(album_id)
-            for track_id in album.track_ids:
-                if self.__cancellable.is_cancelled():
-                    return
-                self.__sync_track_id(Track(track_id))
-
-    def __sync_playlists(self, playlist_ids):
-        """
-            Sync file from playlist to device
-            @param playlist_ids as [int]
-        """
-        for playlist_id in playlist_ids:
-            if self.__cancellable.is_cancelled():
-                break
-            m3u = None
-            stream = None
-            playlist = escape(App().playlists.get_name(playlist_id))
-            try:
-                # Create playlist
-                m3u = Gio.File.new_for_path(
-                    "/tmp/lollypop_%s.m3u" % (playlist,))
-                self.__retry(
-                    m3u.replace_contents,
-                    (b"#EXTM3U\n",
-                     None,
-                     False,
-                     Gio.FileCreateFlags.REPLACE_DESTINATION,
-                     None))
-                stream = m3u.open_readwrite(None)
-            except Exception as e:
-                Logger.error("DeviceWidget::__sync_playlists(): %s" % e)
-            if App().playlists.get_smart(playlist_id):
-                request = App().playlists.get_smart_sql(playlist_id)
-                track_ids = App().db.execute(request)
-            else:
-                track_ids = App().playlists.get_track_ids(playlist_id)
-            # Start copying
-            for track_id in track_ids:
-                if self.__cancellable.is_cancelled():
-                    break
-                if track_id is None:
-                    self.__done += 1
-                    continue
-                track = Track(track_id)
-                (track_name, artists, album_name, is_compilation) =\
-                    self.__sync_track_id(track)
-                if stream is not None:
-                    if is_compilation:
-                        line = "%s/%s\n" %\
-                            (album_name,
-                             track_name)
-                    else:
-                        line = "%s_%s/%s\n" %\
-                            (artists,
-                             album_name,
-                             track_name)
-                    self.__retry(stream.get_output_stream().write,
-                                 (line.encode(encoding="UTF-8"), None))
-            if stream is not None:
-                stream.close()
-            if m3u is not None:
-                dst = Gio.File.new_for_uri(
-                    self.__uri + "/" + playlist + ".m3u")
-                self.__retry(m3u.move,
-                             (dst, Gio.FileCopyFlags.OVERWRITE, None, None))
-
-    def __remove_from_device(self, playlist_ids):
-        """
-            Delete files not available in playlist
-            @param playlist_ids as [int]
-        """
-        # Delete unwanted files on mtp device
-        for uri in self.__on_mtp_files:
-            if self.__cancellable.is_cancelled():
-                return
-            Logger.debug("MtpSync::__remove_from_device(): deleting %s" %
-                         uri)
-            to_delete = Gio.File.new_for_uri(uri)
-            self.__retry(to_delete.delete, (None,))
-            self.__mtp_syncdb.delete_uri(uri)
-            self.__done += 1
-            GLib.idle_add(self.emit, "sync-progress",
-                          self.__done / self.__total)
+                src.copy(dst, Gio.FileCopyFlags.OVERWRITE, None, None)
+            self.__mtp_syncdb.set_mtime(dst_uri, mtime)
 
     def __convert(self, src, dst):
         """
