@@ -73,6 +73,8 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         TracksView.__init__(self, view_type)
         if view_type & ViewType.DND:
             DNDRow.__init__(self)
+        self.__next_row = None
+        self.__previous_row = None
         self.__revealer = None
         self.__parent = parent
         self.__reveal = reveal
@@ -91,9 +93,9 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         """
         if self.get_child() is not None:
             return
-        self._artwork = App().art_helper.get_image(ArtSize.MEDIUM,
-                                                   ArtSize.MEDIUM,
-                                                   "small-cover-frame")
+        self._artwork = Gtk.Image.new()
+        App().art_helper.set_frame(self._artwork, "small-cover-frame",
+                                   ArtSize.MEDIUM, ArtSize.MEDIUM)
         self._artwork.set_margin_start(self.__MARGIN)
         # Little hack: we do not set margin_bottom because already set by
         # get_best_height(): we are Align.FILL
@@ -251,6 +253,36 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
                                            ArtBehaviour.CROP_SQUARE,
                                            self.__on_album_artwork)
 
+    def set_next_row(self, row):
+        """
+            Set next row
+            @param row as Row
+        """
+        self.__next_row = row
+
+    def set_previous_row(self, row):
+        """
+            Set previous row
+            @param row as Row
+        """
+        self.__previous_row = row
+
+    @property
+    def next_row(self):
+        """
+            Get next row
+            @return row as Row
+        """
+        return self.__next_row
+
+    @property
+    def previous_row(self):
+        """
+            Get previous row
+            @return row as Row
+        """
+        return self.__previous_row
+
     @property
     def parent(self):
         """
@@ -422,7 +454,7 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
             popover = self.get_ancestor(Gtk.Popover)
             if popover is not None:
                 popover.popdown()
-            App().window.container.show_artist_view(self._album.artist_ids)
+            App().window.container.show_artists_albums(self._album.artist_ids)
         elif self.__view_type & ViewType.DND:
             if App().player.current_track.album.id == self._album.id:
                 # If not last album, skip it
@@ -724,7 +756,7 @@ class AlbumsListView(LazyLoadingView, ViewController):
                 children[0].populate()
                 children[0].reveal(True)
             else:
-                GLib.idle_add(self.lazy_loading)
+                self.lazy_loading()
             if self._viewport.get_child() is None:
                 self._viewport.add(self._box)
 
