@@ -42,7 +42,9 @@ class ArtistViewSmall(View, ArtistViewCommon):
         album_ids = App().albums.get_ids(artist_ids, genre_ids)
         self.__album_box = AlbumsBoxView(genre_ids,
                                          artist_ids,
-                                         ViewType.MEDIUM | ViewType.SCROLLED)
+                                         ViewType.MEDIUM |
+                                         ViewType.SCROLLED |
+                                         ViewType.NOT_ADAPTIVE)
         height = self._banner.default_height // 3
         self._banner.set_height(height)
         self.__album_box.set_margin_top(height)
@@ -59,6 +61,7 @@ class ArtistViewSmall(View, ArtistViewCommon):
             Connect signals and set active ids
             @param widget as Gtk.Widget
         """
+        View._on_map(self, widget)
         App().settings.set_value("state-one-ids",
                                  GLib.Variant("ai", self._genre_ids))
         App().settings.set_value("state-two-ids",
@@ -66,6 +69,18 @@ class ArtistViewSmall(View, ArtistViewCommon):
         App().settings.set_value("state-three-ids",
                                  GLib.Variant("ai", []))
 
+    def _on_adaptive_changed(self, window, status):
+        """
+            Update banner style
+            @param window as Window
+            @param status as bool
+        """
+        if not status:
+            App().window.container.show_view(
+                self._genre_ids, self._artist_ids, True)
+            # Destroy after any animation
+            GLib.idle_add(self.destroy, priority=GLib.PRIORITY_LOW)
+
 #######################
 # PRIVATE             #
 #######################
