@@ -12,7 +12,7 @@
 
 from gi.repository import Gtk, Pango, GObject
 
-from lollypop.define import App, ArtSize, Type, ArtBehaviour
+from lollypop.define import App, ArtSize, Type, ArtBehaviour, ViewType
 from lollypop.utils import on_query_tooltip
 from lollypop.helper_overlay_radio import OverlayRadioHelper
 
@@ -26,90 +26,109 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
         "overlayed": (GObject.SignalFlags.RUN_FIRST, None, (bool,))
     }
 
-    ctx = App().window.get_pango_context()
-    layout = Pango.Layout.new(ctx)
-    layout.set_text("a", 1)
-    # * 2 => two labels
-    LABEL_HEIGHT = int(layout.get_pixel_size()[1])
-
-    def __init__(self, radio_id, radios):
+    def __init__(self, radio_id, radios, view_type, font_height):
         """
             Init radio widget
             @param radio_id as int
-            @param label_height as int
             @param radios as Radios
+            @param view_type as ViewType
+            @param font_height as int
         """
         Gtk.FlowBoxChild.__init__(self)
-        OverlayRadioHelper.__init__(self)
-        self._widget = None
+        OverlayRadioHelper.__init__(self, radios)
+        self.__widget = None
         self._artwork = None
-        self._watch_loading = True
-        self.__art_size = 120\
-            if App().window.is_adaptive\
-            else ArtSize.BIG
-        self.set_size_request(self.__art_size,
-                              self.__art_size + self.LABEL_HEIGHT)
+        self.__font_height = font_height
         self._radio_id = radio_id
-        self._radios = radios
+        self.__radios = radios
+        self.__name = self.__radios.get_name(self._radio_id)
+        self.__view_type = view_type
+        self._watch_loading = True
+        self.set_view_type(view_type)
 
     def populate(self):
         """
             Init widget content
         """
-        name = self._radios.get_name(self._radio_id)
-        self._widget = Gtk.EventBox()
-        grid = Gtk.Grid()
-        grid.set_orientation(Gtk.Orientation.VERTICAL)
-        self._artwork = App().art_helper.get_image(self.__art_size,
-                                                   self.__art_size,
-                                                   "cover-frame")
-        self.__label = Gtk.Label.new()
-        self.__label.set_justify(Gtk.Justification.CENTER)
-        self.__label.set_ellipsize(Pango.EllipsizeMode.END)
-        self.__label.set_text(name)
-        self.__label.set_property("has-tooltip", True)
-        self.__label.connect("query-tooltip", on_query_tooltip)
-        self._widget.add(grid)
-        self._overlay = Gtk.Overlay.new()
-        self._overlay.add(self._artwork)
-        grid.add(self._overlay)
-        grid.add(self.__label)
-        self.add(self._widget)
-        self.set_artwork()
-        self.set_selection()
-        self.show_all()
-        self._lock_overlay = False
-        self.set_property("halign", Gtk.Align.CENTER)
-        self.set_property("valign", Gtk.Align.CENTER)
-        self._widget.connect("enter-notify-event", self._on_enter_notify)
-        self._widget.connect("leave-notify-event", self._on_leave_notify)
+        if self._artwork is None:
+            self.__widget = Gtk.EventBox()
+            grid = Gtk.Grid()
+            grid.set_orientation(Gtk.Orientation.VERTICAL)
+            self._artwork = Gtk.Image.new()
+            self.__label = Gtk.Label.new()
+            self.__label.set_justify(Gtk.Justification.CENTER)
+            self.__label.set_ellipsize(Pango.EllipsizeMode.END)
+            self.__label.set_text(self.__name)
+            self.__label.set_property("has-tooltip", True)
+            self.__label.connect("query-tooltip", on_query_tooltip)
+            self.__widget.add(grid)
+            self._overlay = Gtk.Overlay.new()
+            self._overlay.add(self._artwork)
+            grid.add(self._overlay)
+            grid.add(self.__label)
+            self.add(self.__widget)
+            self.set_artwork()
+            self.set_selection()
+            self.show_all()
+            self._lock_overlay = False
+            self.set_property("halign", Gtk.Align.CENTER)
+            self.set_property("valign", Gtk.Align.CENTER)
+            self.__widget.connect("enter-notify-event", self._on_enter_notify)
+            self.__widget.connect("leave-notify-event", self._on_leave_notify)
+        else:
+            self.set_artwork()
+
+    def disable_artwork(self):
+        """
+            Disable widget artwork
+        """
+        if self._artwork is not None:
+            self._artwork.set_size_request(self.__art_size, self.__art_size)
+            self._artwork.set_from_surface(None)
 
     def set_artwork(self):
         """
-            Set album artwork
+            Set artwork
         """
-        if self._widget is None:
+        if self.__widget is None:
             return
-        name = self._radios.get_name(self._radio_id)
-        App().art_helper.set_radio_artwork(name,
+        if self.__art_size < ArtSize.BIG:
+            frame = "small-cover-frame"
+        else:
+            frame = "cover-frame"
+        App().art_helper.set_frame(self._artwork,
+                                   frame,
+                                   self.__art_size,
+                                   self.__art_size)
+        App().art_helper.set_radio_artwork(self.__name,
                                            self.__art_size,
                                            self.__art_size,
                                            self._artwork.get_scale_factor(),
-                                           ArtBehaviour.CACHE,
+                                           ArtBehaviour.CACHE |
+                                           ArtBehaviour.CROP,
                                            self.__on_radio_artwork)
 
-    def set_sensitive(self, b):
+    def set_view_type(self, view_type):
         """
-            Ignore set sensitive
+            Update artwork size
+            @param view_type as ViewType
         """
-        pass
+        self.__view_type = view_type
+        if self.__view_type & ViewType.SMALL:
+            self.__art_size = ArtSize.LARGE
+        elif self.__view_type & ViewType.MEDIUM:
+            self.__art_size = ArtSize.BANNER
+        else:
+            self.__art_size = ArtSize.BIG
+        self.set_size_request(self.__art_size,
+                              self.__art_size + self.__font_height)
 
     def do_get_preferred_width(self):
         """
             Return preferred width
             @return (int, int)
         """
-        if self._widget is None:
+        if self.__widget is None:
             return (0, 0)
         width = Gtk.FlowBoxChild.do_get_preferred_width(self)[0]
         return (width, width)
@@ -119,6 +138,7 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
             Set radio name
             @param name as str
         """
+        self.__name = name
         self.__label.set_label(name)
 
     def set_selection(self):
@@ -127,9 +147,8 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
         """
         if self._artwork is None:
             return
-        name = self._radios.get_name(self._radio_id)
         selected = App().player.current_track.id == Type.RADIOS and\
-            name == App().player.current_track.radio_name
+            self.__name == App().player.current_track.radio_name
         if selected:
             self._artwork.set_state_flags(Gtk.StateFlags.SELECTED, True)
         else:
@@ -162,7 +181,7 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
         """
             @return name as str
         """
-        return self._radios.get_name(self._radio_id)
+        return self.__name
 
 #######################
 # PROTECTED           #
