@@ -43,9 +43,11 @@ class PlaylistRow(Row, DNDRow):
         self._grid.insert_column(0)
         self._grid.insert_column(1)
         self._grid.attach(self._indicator, 1, 1, 1, 2)
-        self.__artwork = App().art_helper.get_image(ArtSize.MEDIUM,
-                                                    ArtSize.MEDIUM,
-                                                    "small-cover-frame")
+        self.__artwork = Gtk.Image.new()
+        App().art_helper.set_frame(self.__artwork,
+                                   "small-cover-frame",
+                                   ArtSize.MEDIUM,
+                                   ArtSize.MEDIUM)
         self.__artwork.set_no_show_all(True)
         self.__artwork.set_margin_top(2)
         self.__artwork.set_margin_start(2)
@@ -87,7 +89,8 @@ class PlaylistRow(Row, DNDRow):
             Set previous row
             @param row as Row
         """
-        DNDRow.set_previous_row(self, row)
+        if self._view_type & ViewType.DND:
+            Row.set_previous_row(self, row)
         self.update_artwork_state()
 
     def update_artwork_state(self):
@@ -150,7 +153,8 @@ class PlaylistRow(Row, DNDRow):
             @param widget as Gtk.Widget
         """
         Row._on_destroy(self, widget)
-        DNDRow._on_destroy(self, widget)
+        if self._view_type & ViewType.DND:
+            DNDRow._on_destroy(self, widget)
         self.__artwork = None
 
 #######################
